@@ -59,9 +59,31 @@
 
 #include "radeon_pci_chipset_gen.h"
 
+#include "radeon_chipinfo_gen.h"
 
 #ifdef XSERVER_LIBPCIACCESS
 #include "radeon_pci_device_match_gen.h"
+
+static Bool radeon_ums_supported(ScrnInfoPtr pScrn, struct pci_device *pci_dev)
+{
+    unsigned family = 0, i;
+
+    for (i = 0; i < sizeof(RADEONCards) / sizeof(RADEONCardInfo); i++) {
+        if (pci_dev->device_id == RADEONCards[i].pci_device_id) {
+            family = RADEONCards[i].chip_family;
+            break;
+        }
+    }
+
+    if (family >= CHIP_FAMILY_SUMO) {
+        xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, 0,
+                       "GPU only supported with KMS, using vesa instead.\n");
+        return FALSE;
+    }
+    return TRUE;
+}
+#else
+#define radeon_ums_supported(x, y) TRUE
 #endif
 
 #ifndef XSERVER_LIBPCIACCESS
@@ -131,8 +153,13 @@ radeon_get_scrninfo(int entity_num, void *pci_dev)
         return FALSE;
 
     if (pci_dev) {
-      if (radeon_kernel_mode_enabled(pScrn, pci_dev))
+      if (radeon_kernel_mode_enabled(pScrn, pci_dev)) {
 	kms = 1;
+      } else {
+        if (!radeon_ums_supported(pScrn, pci_dev)) {
+          return FALSE;
+        }
+      }
     }
 
     pScrn->driverVersion = RADEON_VERSION_CURRENT;
